@@ -23,6 +23,14 @@
 
 using namespace std;
 
+#ifdef BUSTER
+#define LOCATE_DB "/var/lib/mlocate/mlocate.db"
+#define UPDATEDB "updatedb.mlocate"
+#else
+#define LOCATE_DB "/var/lib/plocate/plocate.db"
+#define UPDATEDB "updatedb.plocate"
+#endif
+
 /*
 static int read_mounts(const vector<string>& prunefs, vector<string>& mounts)
 {
@@ -64,11 +72,7 @@ static bool updatedb()
 {
 	/* return value is meant as an are_we_up_to_date flag
 	   when running as non-root */
-	#ifdef BUSTER
-	const string db = "/var/lib/mlocate/mlocate.db";
-	#else
-	const string db = "/var/lib/plocate/plocate.db";
-	#endif
+	const string db = LOCATE_DB;
 
 	int rc_locate, rc_dpkg;
 	struct stat stat_locate, stat_dpkg;
@@ -85,7 +89,7 @@ static bool updatedb()
 
 	if (getuid()) return false;
 
-	if (system("updatedb")) {
+	if (system(UPDATEDB)) {
 		cerr << "updatedb failed\n";
 		exit(1);
 	}
@@ -94,6 +98,13 @@ static bool updatedb()
 
 static void one_file(const string& path)
 {
+	// is it a static file ?
+	dpkg_start();
+	if (query(path.c_str())) {
+		dpkg_end();
+		return;
+	}
+
 	string infile = path;
 	if ((path.rfind("/bin/", 0) == 0)
          or (path.rfind("/lib/", 0) == 0)
@@ -106,8 +117,7 @@ static void one_file(const string& path)
 	vector<string> packages;
 	vector<string> dpkg;
 	read_dpkg(packages, dpkg, false);
-	// is it a static file ?
-	// TODO
+	dpkg_end();
 
 	// is it a dynamic file ?
 	vector<owner> globs;
@@ -132,6 +142,21 @@ static void one_file(const string& path)
 	cerr << "no matching package found\n";
 }
 
+static int one_package(const string& package)
+{
+	vector<string> packages;
+	packages.push_back(package);
+	vector<owner> cruft;
+	read_filters("/etc/cruft/filters/", "/usr/share/cruft/ruleset", packages, cruft);
+	packages.push_back("binfmt-support");
+	packages.push_back("ucf");
+	read_explain("/etc/cruft/explain/", packages, cruft);
+	for (const auto& cr: cruft) {
+		if (cr.package == package) cout << cr.path << '\n';
+	}
+	return 0;
+}
+
 static clock_t beg = clock();
 
 static void elapsed(const string& action)
@@ -156,6 +181,7 @@ static void print_help_message()
 	cout << "if not, the whole system is analysed\n\n";
 
 	cout << "OPTIONS\n";
+	cout << "    -p --package     list volatile files only for this one package\n";
 	cout << "    -E --explain     directory for explain scripts (default: " << default_explain_dir << ")\n";
 	cout << "    -F --filter      directory for filters (default: " << default_filter_dir << ")\n";
 	cout << "    -I --ignore      path for ignore file (default: " << default_ignore_file << ")\n";
@@ -167,91 +193,14 @@ static void print_help_message()
 	cout << "    -h --help        this help message\n";
 }
 
-int main(int argc, char *argv[])
+static void cruft(const string& ignore_file,
+                  const string& filter_dir,
+                  const string& ruleset_file,
+                  const string& explain_dir,
+                  const string& bugs_file)
 {
 	bool debug = getenv("DEBUG") != nullptr;
 
-	string explain_dir = default_explain_dir;
-	string filter_dir = default_filter_dir;
-	string ignore_file = default_ignore_file;
-	string ruleset_file = default_ruleset_file;
-	string bugs_file = default_bugs_file;
-
-	const struct option long_options[] =
-	{
-		{"help", no_argument, nullptr, 'h'},
-		{"explain", required_argument, nullptr, 'E'},
-		{"filter", required_argument, nullptr, 'F'},
-		{"ignore", required_argument, nullptr, 'I'},
-		{"ruleset", required_argument, nullptr, 'R'},
-		{"bugs", required_argument, nullptr, 'B'},
-		{0, 0, 0, 0}
-	};
-
-	int opt, opti = 0;
-	while ((opt = getopt_long(argc, argv, "E:F:hI:R:B:", long_options, &opti)) != 0) {
-		if (opt == EOF)
-			break;
-
-		switch (opt) {
-		case 'E':
-			explain_dir = optarg;
-			if (!explain_dir.empty() && explain_dir.back() != '/')
-				explain_dir += '/';
-			break;
-
-		case 'F':
-			filter_dir = optarg;
-			if (!filter_dir.empty() && filter_dir.back() != '/')
-				filter_dir += '/';
-			break;
-
-		case 'h':
-			print_help_message();
-			exit(0);
-
-		case 'I':
-			ignore_file = optarg;
-			break;
-
-		case 'R':
-			ruleset_file = optarg;
-			break;
-
-		case 'B':
-			bugs_file = optarg;
-			break;
-
-		case '?':
-			print_help_message();
-			exit(1);
-
-        default:
-            cerr << "Invalid getopt return value: " << opt << "\n";
-			break;
-        }
-    }
-
-    if (optind < argc) {
-		if (optind + 1 == argc) {
-			struct stat buffer;
-			if (stat(argv[1], &buffer) == 0) {
-				one_file(argv[1]);
-				exit(0);
-			} else {
-				cerr << "file not found\n";
-				exit(1);
-			}
-		}
-
-		cerr << "Invalid non-option arguments:";
-        while (optind < argc)
-			cerr << " " << argv[optind++];
-		cerr << '\n';
-		print_help_message();
-		exit(1);
-    }
-
 	const int SIZEBUF = 200;
 	char buf[SIZEBUF];
 	time_t rawtime;
@@ -273,13 +222,15 @@ int main(int argc, char *argv[])
 
 	vector<string> packages;
 	vector<string> dpkg;
+	dpkg_start();
 	thread thr_dpkg(read_dpkg, ref(packages), ref(dpkg), false);
 	thr_plocate.join();
 	thr_dpkg.join();
+	dpkg_end();
 	elapsed("plocate + dpkg");
 
 	map<string, bug> bugs;
-	read_bugs(bugs, "bugs");
+	read_bugs(bugs, bugs_file);
 
 	// match two main data sources
 	vector<string> cruft;
@@ -384,5 +335,102 @@ int main(int argc, char *argv[])
 	}
 
 	cout << "\nend.\n";
-	return 0;
+	exit(0);
+}
+
+int main(int argc, char *argv[])
+{
+	bool do_one_package = false;
+	string package = "";
+	string explain_dir = default_explain_dir;
+	string filter_dir = default_filter_dir;
+	string ignore_file = default_ignore_file;
+	string ruleset_file = default_ruleset_file;
+	string bugs_file = default_bugs_file;
+
+	const struct option long_options[] =
+	{
+		{"help", no_argument, nullptr, 'h'},
+		{"package", required_argument, nullptr, 'p'},
+		{"explain", required_argument, nullptr, 'E'},
+		{"filter", required_argument, nullptr, 'F'},
+		{"ignore", required_argument, nullptr, 'I'},
+		{"ruleset", required_argument, nullptr, 'R'},
+		{"bugs", required_argument, nullptr, 'B'},
+		{0, 0, 0, 0}
+	};
+
+	int opt, opti = 0;
+	while ((opt = getopt_long(argc, argv, "p:E:F:hI:R:B:", long_options, &opti)) != 0) {
+		if (opt == EOF)
+			break;
+
+		switch (opt) {
+		case 'p':
+			do_one_package = true;
+			package = optarg;
+			break;
+		case 'E':
+			explain_dir = optarg;
+			if (!explain_dir.empty() && explain_dir.back() != '/')
+				explain_dir += '/';
+			break;
+
+		case 'F':
+			filter_dir = optarg;
+			if (!filter_dir.empty() && filter_dir.back() != '/')
+				filter_dir += '/';
+			break;
+
+		case 'h':
+			print_help_message();
+			exit(0);
+
+		case 'I':
+			ignore_file = optarg;
+			break;
+
+		case 'R':
+			ruleset_file = optarg;
+			break;
+
+		case 'B':
+			bugs_file = optarg;
+			break;
+
+		case '?':
+			print_help_message();
+			exit(1);
+
+	        default:
+	            cerr << "Invalid getopt return value: " << opt << "\n";
+				break;
+		}
+	}
+
+
+	if (do_one_package) exit(one_package(package));
+
+	if (optind < argc) {
+		if (optind + 1 == argc) {
+			struct stat buffer;
+			if (stat(argv[1], &buffer) == 0) {
+				one_file(argv[1]);
+				exit(0);
+			} else {
+				cerr << "file not found\n";
+				exit(1);
+			}
+		}
+
+		cerr << "Invalid non-option arguments:";
+		while (optind < argc)
+			cerr << " " << argv[optind++];
+		cerr << '\n';
+		print_help_message();
+		exit(1);
+	}
+
+	// else: standard cruft report
+	cruft(ignore_file, filter_dir, ruleset_file, explain_dir, bugs_file);
 }
